#!/bin/sh

# SVN : $Id: CheckForMissingFiles.sh 15213 2015-09-23 15:26:05Z florin $

# Copyright (C) 2015 MiOS, Ltd., a Hong Kong Corporation
#                     www.mios.com
#            1 - 702 - 4879770 / 866 - 966 - casa
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
exit 0
MIOS_CONSTANTS_FILE="$(dirname $0)/../../mios_constants.sh"
[ -f "$MIOS_CONSTANTS_FILE" ] && source "$MIOS_CONSTANTS_FILE" || echo "FILE-NOT-FOUND: mios_constants.sh"

log_to="file"; log_file="$MIOS_LOGS_PATH/log.mios_firmware"; log_as="check_files"
[ -f "$MIOS_FUNCTIONS_LOG_FILE" ] && source "$MIOS_FUNCTIONS_LOG_FILE" || log "FILE-NOT-FOUND: functions_log.sh"

MiosMountPoint=""
ReqSync=0
ErrorCount=0

main()
{
	get_mount_point || return 1
	check_etc_files
	check_symlinks
	test $ErrorCount == 0
	return $?
}

# Finds where the squashfs is mounted.
get_mount_point()
{
	local miosMtdIdx=$(awk '/"mios"/ {print $1}' /proc/mtd | tr -dc '0-9')
	if [ -z "$miosMtdIdx" ]; then
		log_WARNING "Couldn't find mios mtd. Exit"
		return 1
	fi
	local miosMtdBlock="/dev/mtdblock${miosMtdIdx}"
	MiosMountPoint=$(mount | grep "^$miosMtdBlock" | awk '{print $3}')
	if [ -z "$MiosMountPoint" ]; then
		log_WARNING "Couldn't find the mount point"
		return 1
	fi
	log "Found mount point: $MiosMountPoint"
	return 0
}

# Checks for missing files in /etc and copies them from /mios/etc.
check_etc_files()
{
	# /etc files
	local etcMiosFiles=$(find ${MiosMountPoint}/etc/ \( -type f -o -type l \) \! -name "fresh_install" \! -name "local.mios.com.pem")
	# Other files
	local otherMiosFiles="${MiosMountPoint}/www/index.html"
	#
	local miosFiles="$etcMiosFiles $otherMiosFiles"

	for miosFile in $miosFiles; do
		if [ ! -e "${miosFile#/mios}" ]; then
			log "Restore missing file: ${miosFile#/mios}"
			if ! cp -a "$miosFile" "${miosFile#/mios}" 2>>$log_file; then
				log_ERROR "Failed to copy $miosFile to ${miosFile#/mios}"
				let ErrorCount++
			else
				ReqSync=1
			fi
		fi
	done
	test $ErrorCount == 0
	return $?
}

# Checks for missing symlinks and restores them.
check_symlinks()
{
	local startingErrorCount=$ErrorCount

	# Copy all links, don't link twice.
	miosLinks=$(find $MiosMountPoint -type l | grep -v "^${MiosMountPoint}/etc/")
	for miosLink in $miosLinks; do
		if [ ! -e "${miosLink#/mios}" ]; then
			log "Restore missing symlink: ${miosLink#/mios}"
			if ! cp -a "$miosLink" "${miosLink#/mios}" 2>>$log_file; then
				log_ERROR "Failed to copy $miosLink to ${miosLink#/mios}"
				let ErrorCount++
			else
				ReqSync=1
			fi
		fi
	done

	# Link normal files.
	miosFiles=$(find $MiosMountPoint -type f | grep -v "^${MiosMountPoint}/etc/")
	for miosFile in $miosFiles; do
		if [ ! -e "${miosFile#/mios}" ]; then
			log "Restore missing symlink: ${miosFile#/mios}"
			if ! ln -s "$miosFile" "${miosFile#/mios}" 2>>$log_file; then
				log_ERROR "Failed to create symlink: ${miosFile#/mios} to file: $miosFile"
				let ErrorCount++
			else
				ReqSync=1
			fi
		fi
	done
	test $ErrorCount == $startingErrorCount
	return $?
}

################################################################################

log_START "CheckForMissingFiles $@"
main "$@"
ExitCode=$?
log "END CheckForMissingFiles with exit code $ExitCode"
exit $ExitCode
