#!/bin/sh

# SVN : $Id: Rotate_Logs.sh 19339 2018-05-02 07:28:32Z catalin.marangoci $

# Copyright (C) 2008 Mi Casa Verde, Inc., a Nevada Corporation
#					 www.micasaverde.com
#			1 - 702 - 4879770 / 866 - 966 - casa
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

MIOS_CONSTANTS_FILE="$(dirname $0)/../../mios_constants.sh"
[ -f "$MIOS_CONSTANTS_FILE" ] && source "$MIOS_CONSTANTS_FILE" || echo "FILE-NOT-FOUND: mios_constants.sh"

log_to="file"; log_file=$SCRIPT_LOG_FILE; log_as="rotate_logs"
[ -f "$MIOS_FUNCTIONS_FILE" ] && source "$MIOS_FUNCTIONS_FILE" || echo "FILE-NOT-FOUND: functions.sh"

if ! get_lock -t 1 ; then
	log_WARNING "Another Rotate_Logs process is running. Quit"
	echo "Another Rotate_Logs process is running. Quit"
	exit
fi

FORCE_ROTATE=0
# NetworkMonitor calls this script like this: /usr/bin/Rotate_Logs.sh 1
if [ "$1" == "--force" ] || [ "$1" == "force" ] || [ "$1" == "1" ]; then
	FORCE_ROTATE=1
fi

# needed to enable upload of factory test logs
FACTORY_TEST_INPROGRESS_FILE="/etc/cmh/provisioning"

###############################################################################
#  Check if ntpclient runs and it's set to also set time, not only to get it  #
###############################################################################

NTPPID=$(ps ax | awk '/\/usr\/sbin\/ntpclient/ {print $1}')
if [ -z "${NTPPID}" ] || $(date | grep -q -e ' 200[0-9]$' -e ' 19[0-9][0-9]$'); then
	echo "Starting Ntpclient"
	NEEDSTART=1
else
	for proc in ${NTPPID}; do
		if ! $(cat /proc/${proc}/cmdline  | tr '\000' ' ' | egrep -q ' -s '); then
			echo "Restarting Ntpclient with required params"
			NEEDSTART=1
		fi
	done
fi
if [ "$NEEDSTART" == "1" ]; then
	/etc/init.d/ntpclient restart
fi


MAX_ROTATION_INTERVAL=43200 # 12 hours
MIN_FREE_DISK_SPACE=51200 # KB

# Put here all files from the logs directory which you want to be uploaded to the LOGS server.
TMP_LOG_FILES="log.MiOSRestApi log.upgrade log.mios_services log.Provision log.freshinstall log.RouterMode
			   log.mios_firmware log.Report_AP log.Init-LuaUPnP log.Init-NetworkMonitor log.cmh-ra
			   log.fixNetwork log.WanFailover log.UpgradeZwaveFW log.hotplug-usb log.opkg log.StreamingTunnelsManager"

if [ -f "$FACTORY_TEST_INPROGRESS_FILE" ] && egrep -q "STEP_2_DONE" $FACTORY_TEST_INPROGRESS_FILE; then
	TMP_LOG_FILES="$TMP_LOG_FILES log.FactoryTest log.FactoryTest.debug log.platform_init log.syslog"
fi

pkap=$(get_unit_credential PK_AccessPoint)

numberOfCopies=$(( $(awk -F '[[:blank:]]*=[[:blank:]]*' '/^NumberOfCopies/{ print $2 }' < $MIOS_CMH_CONF_FILE) ))
archiveLogsOnServer=$(( $(awk -F '[[:blank:]]*=[[:blank:]]*' '/^ArchiveLogsOnServer/{ print $2 }' < $MIOS_CMH_CONF_FILE) ))
maxLogsDiskUsage=$(( $(awk -F '[[:blank:]]*=[[:blank:]]*' '/^MaxForRotate/{ print $2 }' < $MIOS_CMH_CONF_FILE) ))

# maxLogsDiskUsage is the number of KB.
if [ $maxLogsDiskUsage -eq 0 ]; then
	# If we don't have a value defined in config for maximum size of the logs,
	# we should allow them up to 50% of the /tmp size.
	maxLogsDiskUsage=$(df | awk '{ if ($6 == "/tmp") printf("%d", $2 * 0.5) }') # KB
	# In case something went wrong and we didn't get the 15%, define a static value.
	[ -z "$maxLogsDiskUsage" ] && maxLogsDiskUsage=131072 # KB
fi

# INFO: It's possible to limit the upload rate using the --limit-rate <speed> curl option.
#if [ -f "$MIOS_USING_3G_FILE" ]; then
#	log_WARNING "We're on 3G, disable file upload"
	archiveLogsOnServer=0
#fi

# In case we have the logs on an USB stick, increase the logs size up to 20 MB.
if grep -qs "$MIOS_CMH_LOGS_PATH\|${MIOS_CMH_LOGS_PATH/var/tmp}" /proc/mounts; then
	log "CMH logs are stored on an USB stick. Increase max size to 20 MB"
	touch /tmp/usb.logging
	maxLogsDiskUsage=20480
fi

now=$(date +%s)
if [ -f /tmp/logrotate_timestamp ]; then
	lastRotation=$(cat /tmp/logrotate_timestamp 2>/dev/null)
else
	lastRotation=$now
	echo "$now" >/tmp/logrotate_timestamp
fi

# If we're out of rotation interval or we're having less that 5 MB free disk space, start log rotation.
freeDiskSpace=$(df | awk '/^tmpfs.*\/tmp$/ {OFMT = "%.0f"; print $4}')
logsDiskUsage=$(du -c $MIOS_CMH_LOGS_PATH/*.log | awk '/total/ {OFMT = "%.0f"; print $1}')
rotationDiff=$((now - lastRotation))

text="PID=$$, force rotate=$FORCE_ROTATE, free space=$freeDiskSpace KB, min=$MIN_FREE_DISK_SPACE KB, cmh logs size=$logsDiskUsage KB, max=$maxLogsDiskUsage KB, now=$now, last rotation=$lastRotation, diff=$((rotationDiff / 60)) min $((rotationDiff % 60)) sec, max=$((MAX_ROTATION_INTERVAL / 3600)) hours"
log "Check logs $text"

# If FORCE_ROTATE==1 or lastRotation was done more that 12 hours ago, then do the rotation anyway.

if [ "$FORCE_ROTATE" != 1 ] && [ $rotationDiff -lt $MAX_ROTATION_INTERVAL ]; then
	if [ $freeDiskSpace -gt $MIN_FREE_DISK_SPACE ]; then
		if [ $logsDiskUsage -lt $maxLogsDiskUsage ]; then
			log "Nothing to do. Quit"
			release_lock
			exit 0
		fi
	fi
fi

######################################
#       Start the log rotation       #
######################################
echo "Start rotating logs $text"
log "Start log rotation"

# Handle lighttpd logs if they are bigger than 350 KB.
lighttpd_error_log="log.lighttpd_error"
if [ $(stat -c %s "$LIGHTTPD_ERROR_LOG_FILE") -gt 358400 ]; then
	echo "===TRUNCATE===" > $LIGHTTPD_ERROR_LOG_FILE
	# Recycle the logs.
	kill -HUP $(pidof lighttpd) 2>/dev/null
fi

# Store some info with this log rotation.
sysinfo_file="$MIOS_CMH_LOGS_PATH/sysinfo.log"
printf "\033[1;41m timestamp:start $(date -R) PID: $$ \033[1;00m \n" >> $sysinfo_file
echo -e "\n=========UPTIME\n" >> $sysinfo_file
uptime >> $sysinfo_file
echo -e "\n=========DF\n" >> $sysinfo_file
df -h >> $sysinfo_file
echo -e "\n=========LOGREAD\n" >> $sysinfo_file
logread | head -300 >> $sysinfo_file
echo -ne "\n" >> $sysinfo_file
# On Vera Lite and Vera 3 logread doesn't support the -l option.
logread -l 300 1>/dev/null 2>&1 && logread -l 300 >> $sysinfo_file || logread | tail -300 >> $sysinfo_file
echo -e "\n=========DMESG\n" >> $sysinfo_file
dmesg >> $sysinfo_file
echo -e "\n=========PS\n" >> $sysinfo_file
ps faux >> $sysinfo_file
echo -e "\n=========TOP\n" >> $sysinfo_file
top -b -n 1 >> $sysinfo_file
echo -e "\n=========IFCONFIG\n" >> $sysinfo_file
ip ad sh | grep -v inet6 >> $sysinfo_file
echo -e "\n=========HOSTS\n" >> $sysinfo_file
cat /etc/hosts >> $sysinfo_file
echo -e "\n=========LOG.ROTATE\n" >> $sysinfo_file
cat $log_file >> $sysinfo_file
rm -f $log_file

printf "\033[1;41m timestamp:stop $(date -R) \033[1;00m \n" >> $sysinfo_file

if [ -f $MIOS_CMH_LOGS_PATH/LuaUPnP.log ]; then
	log "LuaUPnP OK"
	rm -f /tmp/kill_next_time 2>/dev/null
fi

############################
#   Prepare the CMH logs   #
############################
dateStamp=$(date "+%Y-%m-%d_%H-%M-%S")
logs=""
for logFile in $MIOS_CMH_LOGS_PATH/*.log; do
	# e.g. logFile=filnavn.log
	logNumbers=""
	# "for file in ${logFile}[._]*" works well if there is at least one file matching the pattern.
	# If there is no file matching the pattern, $file would have the literal value of the pattern,
	# which is an undesired behavior.
	for oldLogFile in $(ls -1 "${logFile}"[._]* 2>/dev/null); do
		# oldLogFile can look like this:
		# filnavn.log_3.gz  # UI6 and UI7 <= 1.7.1803 / 1.7.775
		# filnavn.log.3.gz  # UI5 and UI7 > 1.7.1803 / 1.7.775
		oldLogFile=${oldLogFile%.gz}
		logNumbers="${logNumbers}${oldLogFile##*[._]} "
	done
	for logNumber in $(echo "$logNumbers" | tr ' ' '\n' | sort -nr); do
		logNumber=${logNumber%.gz}
		if [ "$logNumber" -ge "$numberOfCopies" ]; then
			rm -f "$logFile.$logNumber.gz"
		elif [ "$logNumber" -ge 1 ]; then
			newLogNumber=$((logNumber + 1))
			mv "${logFile}.${logNumber}.gz" "${logFile}.${newLogNumber}.gz" 2>>$log_file
		fi
	done
	if [ $archiveLogsOnServer -eq 1 ]; then
		newLogFile="${logFile}_${dateStamp}"
	else
		newLogFile="${logFile}.1"
	fi
	mv -f "$logFile" "$newLogFile" 2>>$log_file
	logs="$logs"$'\n'"$newLogFile"
done

log "Files in $MIOS_CMH_LOGS_PATH:"
ls -l $MIOS_CMH_LOGS_PATH/* >> $log_file 2>/dev/null

# Signal processes to close and re-open the logs files.
log "Signal processes to close files"
kill -USR2 $(ps ax | grep 'LuaUPnP$\|LuaUPnP $' | awk '{print $1}' | head -n 1) 2>/dev/null
kill -USR2 $(ps ax | grep 'NetworkMonitor$\|NetworkMonitor $' | awk '{print $1}' | head -n 1) 2>/dev/null
kill -USR2 $(pidof 'CamProxyServer') 2>/dev/null
killall tail 2>/dev/null
sleep 3
log "Finished closing files"

if [ ! -f "$MIOS_CMH_LOGS_PATH/LuaUPnP.log" ]; then
	# LuaUPNP must not have gotten the signal.
	# Log what's going on in another file that will get rotated next time, so we can track this.
	ls -l "$MIOS_CMH_LOGS_PATH/LuaUPnP.log" > $MIOS_CMH_LOGS_PATH/FailedRotate.log 2>/dev/null
	top -b -n 1 >> $MIOS_CMH_LOGS_PATH/FailedRotate.log
	echo -e "\nHEAD\n" >> $MIOS_CMH_LOGS_PATH/FailedRotate.log
	head -100 $MIOS_CMH_LOGS_PATH/LuaUPnP.log.$dateStamp >> $MIOS_CMH_LOGS_PATH/FailedRotate.log 2>/dev/null
	echo -e "\nTAIL\n" >> $MIOS_CMH_LOGS_PATH/FailedRotate.log
	tail -100 $MIOS_CMH_LOGS_PATH/LuaUPnP.log.$dateStamp >> $MIOS_CMH_LOGS_PATH/FailedRotate.log 2>/dev/null
fi

################################
#         Upload files         #
################################

for logFile in $logs; do
	log "Processing log file: $logFile"
	rm -f "${logFile}.gz" 2>>$log_file
	gzip "$logFile" 2>>$logFile
	rm -f "$logFile" 2>>$log_file
	ls -al "${logFile}.gz" >> $log_file 2>/dev/null
done

# Handle /tmp logs which are greater than 200k
find $MIOS_LOGS_PATH/ -type f -name "log.*" ! -name "$lighttpd_error_log" -size +200k -maxdepth 1 -exec rm -f {}.gz \; -exec /bin/gzip -f {} \;

release_lock

version=$(cat "$MIOS_VERSION_FILE" 2>>$log_file)

text="Finished log rotation for serial=$pkap, version=$version, date=$dateStamp"

text="ArchiveLogsOnServer is disabled, no logs uploaded"
echo "$text"
log "$text"

echo "$now" >/tmp/logrotate_timestamp
log "Log rotation done at: $dateStamp"
