#!/bin/sh

# SVN : $Id: backup-store.sh 14683 2015-06-24 12:03:38Z florin $

# Copyright (C) 2008 Mi Casa Verde, Inc., a Nevada Corporation
#                     www.micasaverde.com
#            1 - 702 - 4879770 / 866 - 966 - casa
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


# There are 3 ways to do the backup:
# S = Scheduled backup to the server; the user may apply them to another access point.
# D = Download for the end-user; put them in memory for the user to download
# F = Backup before a firmware upgrade; send them to the customer's designated server. We'll delete them after the restore.

MIOS_CONSTANTS_FILE="$(dirname $0)/../../mios_constants.sh"
[ -f "$MIOS_CONSTANTS_FILE" ] && source "$MIOS_CONSTANTS_FILE" || echo "FILE-NOT-FOUND: mios_constants.sh"

log_to="file"; log_file="$MIOS_LOGS_PATH/log.upgrade"; log_as="bs"
[ -f "$MIOS_FUNCTIONS_FILE" ] && source "$MIOS_FUNCTIONS_FILE" || log "FILE-NOT-FOUND: functions.sh"

NOW=$(date +%s)
BACKUP_FILE="/tmp/${NOW}.tgz"
EXISTENT_FILES_FILE="/tmp/existent_files.txt.$$"
SKIP_BACKUP_FILES_FILE="/tmp/skip_bckp_files.txt.$$"
BACKUP_FILES_FILE="/tmp/backup_files.txt.$$"

# These go in the nvram.
# CRITICAL_CONF="$MIOS_PK_AP_FILE $MIOS_HW_KEY_FILE $SYSTEM_OS_RELEASE_FILE /etc/config /etc/firewall.config /etc/passwd /etc/shadow /etc/lighttpd.users"

# There are stored in /etc/cmh-backup and also uploaded to the server.
#NONCRITICAL_CONF="$MIOS_CONF_PATH $MIOS_PLUGINS_DOWNLOAD_PATH $MIOS_RA_PATH $MIOS_BACKUP_INFO_FILE
#	$SYSTEM_TIMEZONE_FILE $SYSTEM_TZ_FULL_FILE $SYSTEM_CRONTABS_FILE /etc/ethers /etc/zwave-aes /etc/dropbear"

# These files and directories won't be included in the backup.
# NOTE #1: KitDevice.json and KitDevice_Variable.json are now in /www/cmh/kit, but when they were added in this script they were in /etc/cmh,
# so keep them here for backward compatibility.
# NOTE #2: ergy_cache_paired_devices.conf can get very big, so we'll upload it separately from cmh-upgrade.sh.
#DONT_BACKUP_CMHCONF_FILES="language language_id last_backup last_reboot platform ra_key scenarios services.conf servers.conf
#	servers.conf.timestamp ui ui_man ui_skin ui_branding users.conf users.conf.timestamp version version_latest servers_whitelist
#	zwave_house_id zwave_products_sys.xml zwave_version network_pnp_sys.xml PK_Account network_pnp.lua KitDevice.json KitDevice_Variable.json
#	ergy_cache_paired_devices.conf"
#DONT_BACKUP_CMHCONF_DIRS="orig"


main()
{
	Method=$1
	if [ "$Method" != "S" ] && [ "$Method" != "D" ] && [ "$Method" != "F" ]; then
		echo "Called with bad options. Valid: {S|D|F}"
		log "backup-store.sh called with bad options. Return"
		return 1
	fi

	local hwKey=$(get_unit_credential HW_KEY)
	if [ -z "$hwKey" ]; then
		echo "No HW_Key: $hwKey. Quit"
		log_ERROR "No HW_Key: $hwKey"
		return 1
	fi

	PK_AccessPoint=$(get_unit_credential PK_AP)
	PK_Account=$(get_unit_credential PK_ACCOUNT)
	RunningVersion=$(cat "$MIOS_VERSION_FILE" 2>/dev/null)

	[ -f "$SYSTEM_OS_RELEASE_FILE" ] && source "$SYSTEM_OS_RELEASE_FILE" || log "FILE-NOT-FOUND: $SYSTEM_OS_RELEASE_FILE"
	[ -z "$DISTRIB_CODENAME" ] && DISTRIB_CODENAME="backfire"

	if [ -f /etc/cmh/fresh_install ]; then
		local fresh_install=1
	else

		local fresh_install=0
	fi
	log "fresh_install=$fresh_install"

	log "Backing up from: $PK_AccessPoint running: $RunningVersion - $DISTRIB_CODENAME in file: $BACKUP_FILE"

	# Create backup info file
	write_backup_info_file

	##################################
	#   Backup non-critical config   #
	##################################

	log "Creating a backup of the non-critical config files"
	cd /

	# - Choose what files to tar.
	# - Remove files which do not exist.
	# - Remove files which should not be backed up.

	# If a file is missing, tar will fail, so we'll skip the missing ones.
#	local nonexistentFiles=""
#	local existentFiles=""

#	for file in $CRITICAL_CONF $NONCRITICAL_CONF; do
#		if [ -f $file ]; then
#			existentFiles="$existentFiles $file"
#		elif [ -d $file ]; then
			# '-print' is required, otherwise the /etc/cmh/persist directory will appear in the output.
			# If '-print' is not added explicitly after '-type', both sides of '-o' will end up printing,
			# whereas if '-print' is used, only that side prints.
#			for file1 in $(find $file/ -path "/etc/cmh/persist" -prune -o -type f -print -o -type l -print); do
#				existentFiles="$existentFiles $file1"
#			done
#		else
#			nonexistentFiles="$nonexistentFiles $file"
#		fi
#	done

#	existentFiles=$(echo $existentFiles | sed 's!/*!/!g; s!\ *! !g' | tr ' ' '\n' | sort -u)
#	log "EXISTENT FILES:$existentFiles\n"
#	echo "$existentFiles" > $EXISTENT_FILES_FILE
#	existentFiles=$(echo "$existentFiles" | tr '\n' ' ')

#	nonexistentFiles=$(echo $nonexistentFiles | sed 's!/*!/!g; s!\ *! !g' | tr ' ' '\n' | sort -u)
#	log "NONEXISTENT FILES:$nonexistentFiles\n"
#	nonexistentFiles=$(echo "$nonexistentFiles" | tr '\n' ' ')

	#######################################
	#   Define what files not to backup   #
	#######################################

	# If this is not a firmware backup, we should not keep the Tech RA files.
#	[ "$Method" != "F" ] && DONT_BACKUP_CMHCONF_FILES="$DONT_BACKUP_CMHCONF_FILES ra_ports ra_password ra_password.lock"

#	local skipBackupFiles=""

#	for file in $DONT_BACKUP_CMHCONF_FILES; do
#		skipBackupFiles="$skipBackupFiles $MIOS_CONF_PATH/$file"
#	done

#	for dir in $DONT_BACKUP_CMHCONF_DIRS; do
#		dir1="$MIOS_CONF_PATH/$dir"
#		for file in $(find $dir1/ -type f -o -type l); do
#			skipBackupFiles="$skipBackupFiles $file"
#		done
#	done

#	skipBackupFiles=$(echo $skipBackupFiles | sed 's!/*!/!g; s!\ *! !g' | tr ' ' '\n' | sort -u)
#	log "SKIP FILES:$skipBackupFiles\n"
#	echo "$skipBackupFiles" > $SKIP_BACKUP_FILES_FILE
#	skipBackupFiles=$(echo "$skipBackupFiles" | tr '\n' ' ')

#	local backupFiles=$(awk 'NR==FNR { a[$0]; next } !($0 in a)' $SKIP_BACKUP_FILES_FILE $EXISTENT_FILES_FILE)
#	echo "$backupFiles" > $BACKUP_FILES_FILE
#	log "BACKUP FILES:\n$backupFiles\n"
#	backupFiles=$(echo "$backupFiles" | tr '\n' ' ')

	# Delete temporary files.
	rm $EXISTENT_FILES_FILE $SKIP_BACKUP_FILES_FILE $BACKUP_FILES_FILE 2>>$log_file

	#############################
	#   Compress backup files   #
	#############################
	log "Compress files"
	log "RunCMD: tar czf $BACKUP_FILE $backupFiles"
	tar czf $BACKUP_FILE /etc/cmh/PK_AccessPoint /etc/cmh/HW_Key /etc/openwrt_release /etc/config /etc/firewall.config /etc/passwd /etc/shadow /etc/lighttpd.users /etc/cmh /etc/cmh-ludl /etc/cmh-ra /etc/TZ /etc/TZ-full /etc/crontabs /etc/mios_backup.info /etc/ethers /etc/zwave-aes /etc/dropbear >>$log_file 2>&1
	log "Compressing done with exit code: $? to file: $(ls -al $BACKUP_FILE)"

	###########################################
	#  Store backup file in /etc/cmh-backup.  #
	###########################################
	if [ "$Method" == "F" ]; then
		mkdir -p $MIOS_BACKUP_PATH 2>>$log_file
		local maxTries=3
		log "Copy backup file: $BACKUP_FILE to: $MIOS_BACKUP_FILE"
		for i in $(seq 1 $maxTries); do
			if cp $BACKUP_FILE $MIOS_BACKUP_FILE 2>>$log_file ; then
				log "File copied OK"
				nvram set backupid="file"
				nvram commit
				sync
				break
			elif [ $i -lt $maxTries ]; then
				log_WARNING "Failed to copy file. Try again [$i/$maxTries]"
				sleep 3
			else
				log_ERROR "Failed to copy file. Return [$i/$maxTries]"
				log "========================== PS =========================="
				ps faxw >>$log_file
				log "========================== DF =========================="
				df >>$log_file
				log "========================================================"
				return 1
			fi
		done
	fi

	if [ "$Method" != "D" ]; then
		upload_backup_file
	else
		mv "$BACKUP_FILE" "/tmp/Backup_download.tgz"
	fi

	if [ "$Method" == "F" ]; then

		#######################################
		#   Backup critical config in nvram   #
		#######################################
		nvram set fresh_install=$fresh_install

		log "Creating a backup of the critical config files in NVRAM"
		cd /

		# Remove non-existent files so that tar doesn't fail.
		EXISTENT_CRITICAL_CONF=""
		for file in $CRITICAL_CONF; do
			if [ ! -f "$file" ] && [ ! -d "$file" ]; then
				log "CRITICAL NON-EXISTENT FILE: $file"
			else
				log "CRITICAL EXISTENT FILE: $file"
				EXISTENT_CRITICAL_CONF="$file $EXISTENT_CRITICAL_CONF"
			fi
		done

		nvram set backup="$(tar zc $EXISTENT_CRITICAL_CONF | uuencode -)"
		nvram commit

		######################################
		# Failsafe check of nvram variabiles #
		######################################
		local reqNvramCommit=0

		if [ -n "$BackupID" ] && [ "$(nvram get backupid)" != "$BackupID" ]; then
			log_WARNING "NVRAM CHECK: Failed to store value for backupid"
			nvram set backupid="$BackupID"
			reqNvramCommit=1
		fi

		if [ -n "$fresh_install" ] && [ "$(nvram get fresh_install)" != "$fresh_install" ]; then
			log_WARNING "NVRAM CHECK: Failed to store NVRAM value for fresh_install"
			nvram set fresh_install="$fresh_install"
			reqNvramCommit=1
		fi

		if [ -n "$EXISTENT_CRITICAL_CONF" ] && [ -z "$(nvram get backup)" ]; then
			log_WARNING "NVRAM CHECK: Failed to store NVRAM value for backup"
			nvram set backup="$(tar zc $EXISTENT_CRITICAL_CONF | uuencode -)"
			reqNvramCommit=1
		else
			log "NVRAM CHECK OK: backup is stored"
		fi

		[ $reqNvramCommit -eq 1 ] && nvram commit

		# TODO: check again if the values were written correctly, and end in an error if they weren't.

		##################
		#   Log values   #
		##################
		log "UPGRADE VALUES from NVRAM:
		-Server_AuthD:        $(nvram get mios_Server_AuthD)
		-Server_AuthD_Alt:    $(nvram get mios_Server_AuthD_Alt)
		-Server_Storage:      $(nvram get mios_Server_Storage)
		-Server_Storage_Alt:  $(nvram get mios_Server_Storage_Alt)
		-BackupID:            $(nvram get backupid)
		-fresh_install:       $(nvram get fresh_install)"
	fi
}


write_backup_info_file()
{
	# Create a file which stores current unit configuration in the backup.
	[ -f "$MIOS_BACKUP_INFO_FILE" ] && mv $MIOS_BACKUP_INFO_FILE ${MIOS_BACKUP_INFO_FILE}.old
	show_unit_info | sed 's/\(^[a-zA-Z]\)/BACKUP_\1/' >> $MIOS_BACKUP_INFO_FILE
	echo "BACKUP_Date=\"$(date -R)\"" >> $MIOS_BACKUP_INFO_FILE
}


##################################
#   Upload backup file to MMS.   #
##################################
upload_backup_file()
{
	[ -f "$MIOS_FUNCTIONS_REST_FILE" ] && source "$MIOS_FUNCTIONS_REST_FILE" || log "FILE-NOT-FOUND: functions_rest.sh"

	### Send non critical backup files to Backup server ###

	local backupSize=$(stat -c %s $BACKUP_FILE 2>>$log_file)
	local backupMd5=$(md5sum $BACKUP_FILE | awk '{print $1}' 2>>$log_file)
	local tags="backup"

	log "Sending backup to server..."
	local restResponseFile
	restResponseFile=$(MiOSRestApi.sh StoreFile \
	                    -p "PK_Device=$PK_AccessPoint" \
	                    -p "PK_Account=${PK_Account:-0}" \
	                    -p "PK_StorageType=$STORAGE_TYPE_BACKUP" \
	                    -p "Format=$FILE_TYPE_TGZ" \
	                    -p "Filesize=$backupSize" \
	                    -p "Md5=$backupMd5" \
	                    -p "Version=$RunningVersion" \
	                    -p "Tags=$tags" \
	                    -p "Data=@$BACKUP_FILE")
	local restExitCode=$?

	[ -s "$restResponseFile" ] && local restResponse=$(cat "$restResponseFile" 2>>$log_file)

	rm -f "$restResponseFile" 2>/dev/null
	rm -f "$BACKUP_FILE" 2>/dev/null

	if [ $restExitCode -ne 0 ] || [ -z "$restResponse" ]; then
		log_ERROR "Failed to upload the backup file, reason: $restResponse, MiOSRestApi exit code: $restExitCode."
		return 1
	fi

	# Sample response: {"PK_Store":232,"Key":"441566506"}
	local PK_Store=$(rest_get_param_json PK_Store "$restResponse")
	local Key=$(rest_get_param_json Key "$restResponse")

	if [ -z "$PK_Store" ] || [ -z "$Key" ]; then
		log_ERROR "Failed to upload the backup file, reason: invalid response from server: $restResponse"
		return 1
	fi

	### Check if the data was sent correctly to the server. ###
	log "Got PK_Store=$PK_Store, Key=$Key; checking if the data was received correctly by the server..."

	# Sample output:
	#	### BEGIN : storage
	#	PK_Store=9932
	#	PK_Device=35002942
	#	Format=tgz
	#	Version=1.6.70
	#	Tags=backup_1361291503
	#	Filesize=15463
	#	Md5=6ebb61da8568721571018d2e6837ec98
	local backupInfoFile
	backupInfoFile=$(MiOSRestApi.sh GetStoredFileInfo -p "Key=$Key" -t "TOKEN_PK_STORE=$PK_Store")
	restExitCode=$?

	if [ $restExitCode -ne 0 ] || [ ! -s "$backupInfoFile" ]; then
		local errorMessage=$(cat "$backupInfoFile" 2>/dev/null)
		rm -f "$backupInfoFile" 2>/dev/null
		log_ERROR "Failed to get backup info from server, MiOSRestApi exit code: $restExitCode, reason: $errorMessage"
		return 1
	fi

	[ -f "$backupInfoFile" ] && source "$backupInfoFile"
	local infoFilesize=$Filesize
	local infoMd5=$Md5
	if [ "$infoFilesize" != "$backupSize" ]; then
		log_ERROR "The uploaded backup file size is not correct. backup_size=$backupSize, info_file_size=$infoFilesize"
		return 1
	elif [ "$infoMd5" != "$backupMd5" ]; then
		log_ERROR "The uploaded backup file MD5 sum is not correct. backup_md5='$backupMd5', info_md5='$infoMd5'"
		return 1
	fi

	log_SUCCESS "Backup uploaded"

	if [ "$Method" == "F" ]; then
		BackupID="${PK_Store}-${Key}"
		nvram set backupid=$BackupID

		# Write the servers in NVRAM.
		Server_Storage=$(get_server Server_Storage)
		Server_Storage_Alt=$(get_server Server_Storage 1)
		if [ -n "$Server_Storage" ]; then
			nvram set mios_Server_Storage="$Server_Storage"
			if [ -n "$Server_Storage_Alt" ] && [ "$Server_Storage_Alt" != "$Server_Storage" ]; then
				nvram set mios_Server_Storage_Alt="$Server_Storage_Alt"
			else
				nvram unset mios_Server_Storage_Alt
			fi
		elif [ -n "$Server_Storage_Alt" ]; then
			nvram set mios_Server_Storage="$Server_Storage_Alt"
			nvram unset mios_Server_Storage_Alt
		fi

		nvram commit
	fi

	return 0
}

# Redefine the log function.
log() {
	echo -e "$(date +%F_%T) -[$$]- ==bs== $* ==bs==" >> $log_file
}

###################################
#              START              #
###################################
log_START "BACKUP STORE: $*"
main "$@"
ExitCode=$?
log "=== END BACKUP STORE ==="
exit $ExitCode
