#!/bin/sh

# SVN : $Id: lighttpd_ssl.sh 14643 2015-06-22 08:30:05Z florin $

# Copyright (C) 2008 Mi Casa Verde, Inc., a Nevada Corporation
#                     www.micasaverde.com
#            1 - 702 - 4879770 / 866 - 966 - casa
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

[ -f "$(dirname $0)/../../mios_constants.sh" ] && source "$(dirname $0)/../../mios_constants.sh"

CERTIFICATE_FILE="/etc/mios/sslcerts/CERTIFICATES/local.mios.com.pem"
TSTAMP20100101=1262304000 # If the date is older than this, generate a new certificate,
                          # otherwise lighttpd might not start.

log() {
	logger -t "lighttpd_ssl" -- "$*"
}

# Check if we must enable SSL for lighttpd.
if [ -f "$MIOS_CMH_CONF_FILE" ]; then
	enableSsl=$(awk -F'=' '/^EnableSSL/ {print $2}' "$MIOS_CMH_CONF_FILE" | tr -d ' \t')
fi

if [ "$enableSsl" == 1 ]; then

	# If the certificate file exists, check if it has expired, or if it will expire in less than a week.
	if [ -s "$CERTIFICATE_FILE" ]; then
		# openssl command sample output: notAfter=Dec 29 00:36:39 2009 GMT
		expDate=$(openssl x509 -enddate -noout -in "$CERTIFICATE_FILE" 2>/dev/null | awk -F'=' '{print $2}')
		expTstamp=$(date -d "$expDate" -D '%b %d %T %Y %Z' -u +'%s')
		nowTstamp=$(date +'%s')
		log "Certificate file already exists, expires on: $expTstamp($expDate), now: $nowTstamp"
		if [ $nowTstamp -lt $TSTAMP20100101 ] || [ $expTstamp -le $((nowTstamp + 604800)) ]; then
			log "Remove certificate file"
			rm -f "$CERTIFICATE_FILE" 1>/dev/null 2>&1
		fi
	fi

	# Check if the certificate file exists, and create one if it doesn't.
	if [ ! -s "$CERTIFICATE_FILE" ]; then
		log "Generate certificate"
		gen_ssl.sh 1>/dev/null 2>&1
	fi

	# Enable SSL if the certificate file exists/was created.
	[ -s "$CERTIFICATE_FILE" ] && cat <<-SSL
		\$SERVER["socket"] == ":443" {
			ssl.engine = "enable"
			ssl.pemfile = "$CERTIFICATE_FILE"
		}
	SSL
exit 0
else
	# SSL is disabled, remove the certificate file.
	[ -e "$CERTIFICATE_FILE" ] && rm -f "$CERTIFICATE_FILE" >/dev/null 2>&1
exit 0
fi
