#!/bin/sh /etc/rc.common

# SVN: $Id: wan_failover.sh 17340 2016-08-30 12:44:20Z florin $

# Copyright (C) 2013 MiOS, Ltd., a Hong Kong Corporation
#                    www.mios.com
#               iulian.vacariu@mios.com
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

# Make it start last process so we're sure all networking is up.
START=999
exit 0

# Skip it if we're on batteries.
if [ -f /tmp/.running_on_batteries ]; then
	echo "Running on batteries. Skipping..."
	exit 0
fi

MIOS_CONSTANTS_FILE="$(dirname $0)/../../mios_constants.sh"
[ -f "$MIOS_CONSTANTS_FILE" ] && source "$MIOS_CONSTANTS_FILE" ||
	{ logger -s -t "FILE-NOT-FOUND" "mios_constants.sh"; }


start()
{
	log_to="file"; log_file="$MIOS_LOGS_PATH/log.WanFailover"; log_as="wan_failover"
	[ -f "$MIOS_FUNCTIONS_FILE" ] && source "$MIOS_FUNCTIONS_FILE" ||
		{ logger -s -t "FILE-NOT-FOUND" "functions.sh"; }

	log_START "Wan failover init script"

	# Create the WAN failover config directory if it doesn't exist.
	mkdir -p $MIOS_WF_CONF_PATH 1>/dev/null 2>&1

	# Don't start wan failover if it's been explicitly disabled.
	local enabled=$(awk -F '=' '/EnableWanFailover/ {print $2}' < $MIOS_CMH_CONF_FILE)
	if [ "$enabled" == 0 ]; then
		log "Wan failover is disabled. Exit"
		exit 0
	fi

	if type ifstatus 1>/dev/null 2>&1 && ifstatus "3g" | grep -q '"up".*true' || ifconfig 2>/dev/null | grep -q "3g-3g"; then
		log "The 3g interface is already up. Start Wan failover"
		Start_WanFailover.sh &
		exit 0
	fi

	# If the 3G interface is not configured, don't start wan failover.

	# Check if the 3G interface exists in the configuration file.
	if ! uci get network.3g >/dev/null 2>&1; then
		log_WARNING "The 3g interface doesn't exist in the config file. Exit"
		exit 0
	fi

	log "3g configuration:\n$(uci show network.3g)"

	# Check if the device is configured.
	if ! uci get network.3g.device 1>/dev/null 2>&1 && ! uci get network.3g.ifname 1>/dev/null 2>&1; then
		log_DIE "'device' or 'ifname' is missing from the interface config."
	fi

	#if ! egrep -qs 'options[[:blank:]]+timeout' /etc/resolv.conf; then
	#	echo 'options timeout:1' >> /etc/resolv.conf
	#	sync
	#fi

	# Send this to background, to speed up the startup process.
	(
		log "Start the 3g interface"
		if type ubus 1>/dev/null 2>&1; then
			ubus call network.interface.3g up 1>>$log_file 2>&1
		else
			ifup "3g" 1>>$log_file 2>&1
		fi
		sleep 10 # It takes more than 5 seconds to set up the 3g interface:
		         # Wed Mar 30 14:52:21 2016 daemon.notice netifd: Interface '3g' is setting up now
		         # Wed Mar 30 14:52:26 2016 daemon.notice netifd: 3g (3865): Trying to set PIN
		local veraModel=$(get_vera_model)
		if [ "$veraModel" == "Vera4Lite" ] || [ "$veraModel" == "Vera5Plus" ]; then
			logread -l 100 | egrep 'chat|pppd|netifd' >>$log_file
		else
			logread | tail -100 | egrep 'chat|pppd|netifd' >>$log_file
		fi
		log "Start the Wan failover daemon"
		Start_WanFailover.sh
	) &
}


stop()
{
	# Stop Wan failover daemon.
	kill $(ps | grep -v "grep\|tail" | grep -F "WanFailover" | awk '{ print $1 }') 1>/dev/null 2>&1
	sleep 2
	kill -9 $(ps | grep -v "grep\|tail" | grep -F "WanFailover" | awk '{ print $1 }') 1>/dev/null 2>&1
}


restart()
{
	stop
	sleep 2
	start
}
